﻿using System;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace MangoClient
{
    public enum StartState
    { FreshStart, TombstoneResurrected, NonTombstoneReactivated }

    public partial class App : Application
    {

        #region Fields/Properties

        private bool phoneApplicationInitialized = false;
        public PhoneApplicationFrame RootFrame { get; private set; }
        public static StartState StartState;
        private bool wasTerminated = true;

        private static MainViewModel viewModel = null;
        public static MainViewModel ViewModel
        {
            get
            {
                // Delay creation of the view model until necessary.
                if (viewModel == null)
                {
                    viewModel = new MainViewModel();
                }
                return viewModel;
            }
        }

        // Additional properties for XNA integration.
        public ContentManager Content { get; private set; }
        public AppServiceProvider Services { get; private set; }

        // Provides access to a GameTimer that is set up to pump the FrameworkDispatcher.
        public GameTimer FrameworkDispatcherTimer { get; private set; }

        private bool isXnaInitialized;

        #endregion


        #region Init

        public App()
        {
            // Global handler for uncaught exceptions. 
            UnhandledException += Application_UnhandledException;

            // Show graphics profiling information while debugging.
            //if (System.Diagnostics.Debugger.IsAttached)
            //{
                //Application.Current.Host.Settings.EnableFrameRateCounter = true;
                //Application.Current.Host.Settings.EnableRedrawRegions = true;
                //Application.Current.Host.Settings.EnableCacheVisualization = true;

                // Disable the application idle detection by setting the UserIdleDetectionMode property of the
                // application's PhoneApplicationService object to Disabled.
                // Caution:- Use this under debug mode only. Application that disables user idle detection will continue to run
                // and consume battery power when the user is not using the phone.
                //PhoneApplicationService.Current.UserIdleDetectionMode = IdleDetectionMode.Disabled;
            //}

            // Standard Silverlight initialization.
            InitializeComponent();

            // Phone-specific initialization.
            InitializePhoneApplication();

            // UNDONE: Defer XNA initialization until the user navigates to the Game page -
            // this will improve startup time.
            // Added for XNA integration.
            //InitializeXnaApplication();
        }

        // Do not add any additional code to this method
        private void InitializePhoneApplication()
        {
            if (phoneApplicationInitialized)
            {
                return;
            }

            // Create the frame but don't set it as RootVisual yet; this allows the splash
            // screen to remain active until the application is ready to render.
            // Use the Toolkit TransitionFrame so that we can support page transition animations.
            //RootFrame = new PhoneApplicationFrame();
            RootFrame = new TransitionFrame();
            RootFrame.Navigated += CompleteInitializePhoneApplication;

            // Handle navigation failures
            RootFrame.NavigationFailed += RootFrame_NavigationFailed;

            // Ensure we don't initialize again
            phoneApplicationInitialized = true;
        }

        // Do not add any additional code to this method
        private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
        {
            // Set the root visual to allow the application to render
            if (RootVisual != RootFrame)
            {
                RootVisual = RootFrame;
            }

            // Remove this handler since it is no longer needed
            RootFrame.Navigated -= CompleteInitializePhoneApplication;
        }

        #endregion


        #region Lifetime events

        // Code to execute when the application is launching (eg, from Start)
        // This code will not execute when the application is reactivated
        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
            // This is the fresh start case:
            // [Closing-->]Ctor-->Launching
            StartState = StartState.FreshStart;

            ViewModel.LoadData();
        }

        // Code to execute when the application is activated (brought to foreground)
        // This code will not execute when the application is first launched
        private void Application_Activated(object sender, ActivatedEventArgs e)
        {
            if (wasTerminated)
            {
                // This is the tombstone/resurrection case:
                // Deactivated-->Ctor-->Activated
                StartState = StartState.TombstoneResurrected;
            }
            else
            {
                // The non-tombstone case:
                // Deactivated-->Activated
                StartState = StartState.NonTombstoneReactivated;
            }

            // Ensure that application state is restored appropriately
            if (!App.ViewModel.IsDataLoaded)
            {
                App.ViewModel.LoadData();
            }
        }

        // Code to execute when the application is deactivated (sent to background)
        // This code will not execute when the application is closing
        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
            // Ensure that required application state is persisted here.
            wasTerminated = false;
        }

        // Code to execute when the application is closing (eg, user hit Back)
        // This code will not execute when the application is deactivated
        private void Application_Closing(object sender, ClosingEventArgs e)
        {
        }

        #endregion


        #region Exceptions

        // Code to execute if a navigation fails
        private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // A navigation has failed; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
            else
            {
                if (e.Exception != null && !String.IsNullOrEmpty(e.Exception.Message))
                {
                    MessageBox.Show(String.Format("Sorry, we've encountered an error: {0}.", e.Exception.Message));
                }
                else
                {
                    MessageBox.Show("Sorry, we've encountered an error.");
                }
                e.Handled = true;
            }
        }

        // Code to execute on Unhandled Exceptions
        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
            else
            {
                if (e.ExceptionObject != null && !String.IsNullOrEmpty(e.ExceptionObject.Message))
                {
                    MessageBox.Show(String.Format("Sorry, we've encountered an error: {0}.", e.ExceptionObject.Message));
                }
                else
                {
                    MessageBox.Show("Sorry, we've encountered an error.");
                }
                e.Handled = true;
            }
        }

        #endregion


        #region XNA application initialization

        // Added for XNA integration.
        // Performs initialization of the XNA types required for the application.
        internal void InitializeXnaApplication()
        {
            if (!isXnaInitialized)
            {
                // Create the service provider
                Services = new AppServiceProvider();

                // Add the SharedGraphicsDeviceManager to the Services as the IGraphicsDeviceService for the app
                foreach (object obj in ApplicationLifetimeObjects)
                {
                    if (obj is IGraphicsDeviceService)
                    {
                        Services.AddService(typeof(IGraphicsDeviceService), obj);
                    }
                }

                // Create the ContentManager so the application can load precompiled assets
                Content = new ContentManager(Services, "Content");

                // Create a GameTimer to pump the XNA FrameworkDispatcher
                FrameworkDispatcherTimer = new GameTimer();
                FrameworkDispatcherTimer.FrameAction += FrameworkDispatcherFrameAction;
                FrameworkDispatcherTimer.Start();

                isXnaInitialized = true;
            }
        }

        // An event handler that pumps the FrameworkDispatcher each frame.
        // FrameworkDispatcher is required for a lot of the XNA events and
        // for certain functionality such as SoundEffect playback.
        private void FrameworkDispatcherFrameAction(object sender, EventArgs e)
        {
            FrameworkDispatcher.Update();
        }

        #endregion

    }
}