﻿using System;
using System.Collections.Generic;
using System.IO.IsolatedStorage;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input.Touch;

namespace MangoClient
{
    public partial class GamePage : PhoneApplicationPage
    {

        #region Properties

        public GraphicsDevice GraphicsDevice { get; set; }

        #endregion


        #region Fields

        private ContentManager contentManager;
        private GameTimer timer;
        private SpriteBatch spriteBatch;
        private Texture2D mainBackground;
        private ParallaxingBackground backgroundLayer1;
        private ParallaxingBackground backgroundLayer2;
        private Texture2D enemyTexture;
        private List<Enemy> enemies;
        private TimeSpan enemySpawnTime;
        private TimeSpan previousSpawnTime;
        private Random random;
        private Texture2D projectileTexture;
        private List<Projectile> projectiles;
        private TimeSpan fireTime;
        private TimeSpan previousFireTime;
        private Texture2D explosionTexture;
        private List<Animation> explosions;
        private SpriteFont font;
        private SoundEffect projectileSound;
        private SoundEffect collisionSound;
        private SoundEffect warningSound;
        private SoundEffect gameOverSound;
        private SoundEffectInstance warningSoundInstance;
        private int score;
        private Player player;
        private int xBounds;
        private int yBounds;
        private bool isPlayerAlive;
        private Vector2 initialPlayerPosition;
        private bool isContentLoaded;
        private Animation playerAnimation;

        #endregion


        #region Init

        public GamePage()
        {
            InitializeComponent();

            ((MangoClient.App)App.Current).InitializeXnaApplication();

            // Create a timer for this page
            timer = new GameTimer();
            timer.UpdateInterval = TimeSpan.FromTicks(333333);
            timer.Update += OnUpdate;
            timer.Draw += OnDraw;

            // Get a private reference to the appp's ContentManager.
            contentManager = (Application.Current as App).Content;
            GraphicsDevice = SharedGraphicsDeviceManager.Current.GraphicsDevice;

            // Enable FreeDrag.
            TouchPanel.EnabledGestures = GestureType.FreeDrag | GestureType.Tap;

            // Initialize the player, background, and enemies and projectiles collections.
            player = new Player();
            backgroundLayer1 = new ParallaxingBackground();
            backgroundLayer2 = new ParallaxingBackground();
            enemies = new List<Enemy>();
            previousSpawnTime = TimeSpan.Zero;
            enemySpawnTime = TimeSpan.FromSeconds(1.0f);
            random = new Random();
            projectiles = new List<Projectile>();
            explosions = new List<Animation>();
            score = 0;
            player.Health = 100;
            fireTime = TimeSpan.FromSeconds(.15f);
            isPlayerAlive = true;

            // Set the starting position of the player around the middle of the screen and to the back.
            initialPlayerPosition = new Vector2(
                GraphicsDevice.Viewport.TitleSafeArea.X, GraphicsDevice.Viewport.TitleSafeArea.Y
                + GraphicsDevice.Viewport.TitleSafeArea.Height / 2);
        }

        #endregion


        #region LoadContent

        public void LoadContent()
        {
            // Load the image and sound content.
            backgroundLayer1.Initialize(contentManager, "backgroundLayer1", GraphicsDevice.Viewport.Height, -1);
            backgroundLayer2.Initialize(contentManager, "backgroundLayer2", GraphicsDevice.Viewport.Height, -2);
            mainBackground = contentManager.Load<Texture2D>("mainbackground");
            enemyTexture = contentManager.Load<Texture2D>("mineAnimation");
            projectileTexture = contentManager.Load<Texture2D>("laser");
            explosionTexture = contentManager.Load<Texture2D>("explosion");
            projectileSound = contentManager.Load<SoundEffect>("sound/projectile");
            collisionSound = contentManager.Load<SoundEffect>("sound/collision");
            warningSound = contentManager.Load<SoundEffect>("sound/warning");
            gameOverSound = contentManager.Load<SoundEffect>("sound/gameover");
            font = contentManager.Load<SpriteFont>("gameFont");
            warningSoundInstance = warningSound.CreateInstance();
            warningSoundInstance.IsLooped = true;

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load the player resources.
            playerAnimation = new Animation();
            Texture2D playerTexture = contentManager.Load<Texture2D>("shipAnimation");

            // Each frame in the animation strip is 150 wide, the strip is 75 high, and there are 5 frames.
            playerAnimation.Initialize(playerTexture, Vector2.Zero, 150, 75, 5, 30, Color.White, 1f, true);

            isContentLoaded = true;
        }

        private void InitializePlayer(Vector2 newPosition)
        {
            player.Initialize(playerAnimation, newPosition);
            xBounds = player.Width / 2;
            yBounds = player.Height / 2;
        }

        #endregion


        #region Manage Explosions

        private void AddExplosion(Vector2 position)
        {
            Animation explosion = new Animation();
            explosion.Initialize(explosionTexture, position, 134, 134, 12, 45, Color.White, 1f, false);
            explosions.Add(explosion);
        }

        private void UpdateExplosions(TimeSpan totalTime, TimeSpan elapsedTime)
        {
            for (int i = explosions.Count - 1; i >= 0; i--)
            {
                explosions[i].Update(totalTime, elapsedTime);
                if (explosions[i].Active == false)
                {
                    explosions.RemoveAt(i);
                }
            }
        }

        #endregion


        #region Manage Enemies

        private void AddEnemy()
        {
            // Initialize the animation object.
            Animation enemyAnimation = new Animation();
            enemyAnimation.Initialize(enemyTexture, Vector2.Zero, 65, 67, 8, 30, Color.White, 1f, true);

            // Randomly generate the position of the enemy, and add the new enemy to the list.
            Vector2 position = new Vector2(GraphicsDevice.Viewport.Width + enemyTexture.Width / 2, random.Next(100, GraphicsDevice.Viewport.Height - 100));
            Enemy enemy = new Enemy();
            enemy.Initialize(enemyAnimation, position);
            enemies.Add(enemy);
        }

        private void UpdateEnemies(TimeSpan totalTime, TimeSpan elapsedTime)
        {
            // Spawn a new enemy every 1.5 seconds.
            if (totalTime - previousSpawnTime > enemySpawnTime)
            {
                previousSpawnTime = totalTime;
                AddEnemy();
            }

            // Update the enemies.
            for (int i = enemies.Count - 1; i >= 0; i--)
            {
                enemies[i].Update(totalTime, elapsedTime);
                if (enemies[i].Active == false)
                {
                    if (enemies[i].Health <= 0)
                    {
                        // Add an explosion, with sound, and increment the score.
                        AddExplosion(enemies[i].Position);
                        collisionSound.Play();
                        score += enemies[i].Value;
                    }
                    enemies.RemoveAt(i);
                }
            }
        }

        #endregion


        #region Manage Projectiles

        private void AddProjectile(Vector2 position)
        {
            Projectile projectile = new Projectile();
            projectile.Initialize(GraphicsDevice.Viewport, projectileTexture, position);
            projectiles.Add(projectile);
        }

        private void UpdateProjectiles()
        {
            for (int i = projectiles.Count - 1; i >= 0; i--)
            {
                projectiles[i].Update();

                if (projectiles[i].Active == false)
                {
                    projectiles.RemoveAt(i);
                }
            }
        }

        #endregion


        #region Navigation Events

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            ((MangoClient.App)App.Current).InitializeXnaApplication();

            // Set the sharing mode of the graphics device to turn on XNA rendering
            SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(true);

            // Always load content, to allow for the case where the user uses a Launcher 
            // (eg, the help/support link which fires up the browser) before selecting
            // this game page: that will come through as a StartState.TombstoneResurrected.
            if (!isContentLoaded)
            {
                LoadContent();
                InitializePlayer(initialPlayerPosition);
            }

            int tmpScore;
            int tmpHealth;
            Vector2 tmpPosition;
            switch (App.StartState)
            {
                case StartState.FreshStart:
                    // FreshStart - no state to retrieve.
                    break;
                case StartState.TombstoneResurrected:
                    // Resurrected - we can retrieve state.
                    if (IsolatedStorageSettings.ApplicationSettings.TryGetValue<int>("score", out tmpScore))
                    {
                        score = tmpScore;
                    }
                    if (IsolatedStorageSettings.ApplicationSettings.TryGetValue<int>("health", out tmpHealth))
                    {
                        player.Health = tmpHealth;
                    }
                    if (IsolatedStorageSettings.ApplicationSettings.TryGetValue<Vector2>("position", out tmpPosition))
                    {
                        player.Position = tmpPosition;
                        InitializePlayer(tmpPosition);
                    }

                    break;
                case StartState.NonTombstoneReactivated:
                    // Reactivated - no need to retrieve state.
                    break;
            }

            timer.Start();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            // Make sure we stop the warning sound. This is for the case where
            // the warning sound is playing when the user hits the Back key.
            if (warningSoundInstance.State == SoundState.Playing)
            {
                warningSoundInstance.Stop();
            }

            // Stop the timer
            timer.Stop();

            // Set the sharing mode of the graphics device to turn off XNA rendering
            SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(false);

            // Persist settings between the game and the main application.
            IsolatedStorageSettings.ApplicationSettings["score"] = score;
            IsolatedStorageSettings.ApplicationSettings["health"] = player.Health;
            IsolatedStorageSettings.ApplicationSettings["position"] = player.Position;
        }

        #endregion


        #region Update

        private void OnUpdate(object sender, GameTimerEventArgs e)
        {
            // Update the parallaxing background even when the game is over.
            backgroundLayer1.Update();
            backgroundLayer2.Update();

            // Update the player, enemies, collision, projectiles and explosions
            // only when the player is still alive.
            if (isPlayerAlive)
            {
                UpdatePlayer(e.TotalTime, e.ElapsedTime);
                UpdateEnemies(e.TotalTime, e.ElapsedTime);
                UpdateCollision();
                UpdateProjectiles();
                UpdateExplosions(e.TotalTime, e.ElapsedTime);
            }
        }

        private void UpdatePlayer(TimeSpan totalTime, TimeSpan elapsedTime)
        {
            player.Update(totalTime, elapsedTime);

            // Windows Phone Controls.
            while (TouchPanel.IsGestureAvailable)
            {
                GestureSample gesture = TouchPanel.ReadGesture();
                if (gesture.GestureType == GestureType.FreeDrag)
                {
                    player.Position += gesture.Delta;
                }
            }

            // Make sure that the player does not go out of bounds.
            player.Position.X = MathHelper.Clamp(player.Position.X, xBounds, GraphicsDevice.Viewport.Width - xBounds);
            player.Position.Y = MathHelper.Clamp(player.Position.Y, yBounds, GraphicsDevice.Viewport.Height - yBounds);

            // Fire only every interval we set as the fireTime.
            if (totalTime - previousFireTime > fireTime)
            {
                // Reset our current time, and add the projectile to the front/center of the player:
                previousFireTime = totalTime;
                AddProjectile(player.Position + new Vector2(player.Width / 2, 0));
                projectileSound.Play();
            }

            // Warning: health approaching zero.
            if (player.Health <= 10 && warningSoundInstance.State != SoundState.Playing)
            {
                warningSoundInstance.Play();
            }

            // Game over.
            if (player.Health <= 0)
            {
                warningSoundInstance.Stop();
                gameOverSound.Play();
                isPlayerAlive = false;
            }
        }

        private void UpdateCollision()
        {
            // Use the Rectangle's built-in intersect function to 
            // determine if two objects are overlapping.
            Rectangle rectangle1;
            Rectangle rectangle2;

            // Only create the rectangle once for the player.
            rectangle1 = new Rectangle(
                (int)player.Position.X, (int)player.Position.Y, player.Width, player.Height);

            // Do the collision between the player and the enemies.
            for (int i = 0; i < enemies.Count; i++)
            {
                rectangle2 = new Rectangle(
                    (int)enemies[i].Position.X, (int)enemies[i].Position.Y, enemies[i].Width, enemies[i].Height);

                // Determine if the two objects collided with each other.
                if (rectangle1.Intersects(rectangle2))
                {
                    // Subtract the health from the player based on the enemy damage.
                    player.Health -= enemies[i].Damage;

                    // Since the enemy collided with the player destroy it.
                    enemies[i].Health = 0;

                    // If the player health is less than zero we died.
                    if (player.Health <= 0)
                    {
                        player.Active = false;
                    }
                }
            }

            // Projectile vs Enemy Collision.
            for (int i = 0; i < projectiles.Count; i++)
            {
                for (int j = 0; j < enemies.Count; j++)
                {
                    // Create the rectangles we need to determine if we collided with each other.
                    rectangle1 = new Rectangle((int)projectiles[i].Position.X -
                        projectiles[i].Width / 2, (int)projectiles[i].Position.Y -
                        projectiles[i].Height / 2, projectiles[i].Width, projectiles[i].Height);

                    rectangle2 = new Rectangle((int)enemies[j].Position.X - enemies[j].Width / 2,
                        (int)enemies[j].Position.Y - enemies[j].Height / 2, enemies[j].Width, enemies[j].Height);

                    // Determine if the two objects collided with each other.
                    if (rectangle1.Intersects(rectangle2))
                    {
                        enemies[j].Health -= projectiles[i].Damage;
                        projectiles[i].Active = false;
                    }
                }
            }
        }

        #endregion


        #region Draw

        private void OnDraw(object sender, GameTimerEventArgs e)
        {
            GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin();
            DrawBackground();

            if (isPlayerAlive)
            {
                // Draw the player, enemies, projectiles and explosions.
                player.Draw(spriteBatch);
                for (int i = 0; i < enemies.Count; i++)
                {
                    enemies[i].Draw(spriteBatch);
                }
                for (int i = 0; i < projectiles.Count; i++)
                {
                    projectiles[i].Draw(spriteBatch);
                }
                for (int i = 0; i < explosions.Count; i++)
                {
                    explosions[i].Draw(spriteBatch);
                }

                // Draw the score and player health.
                spriteBatch.DrawString(font, "mangotini score:  " + score, new Vector2(
                    GraphicsDevice.Viewport.TitleSafeArea.X + 12, GraphicsDevice.Viewport.TitleSafeArea.Y), Color.Orange);
                spriteBatch.DrawString(font, "field strength:  " + player.Health, new Vector2(
                    GraphicsDevice.Viewport.TitleSafeArea.X + 12, GraphicsDevice.Viewport.TitleSafeArea.Y + 36), Color.Orange);
            }
            else
            {
                spriteBatch.DrawString(font, "game over!", new Vector2(
                    GraphicsDevice.Viewport.X + 150,
                    GraphicsDevice.Viewport.Y + 340), Color.Orange);
                spriteBatch.DrawString(font, "score:  " + score, new Vector2(
                    GraphicsDevice.Viewport.X + 150,
                    GraphicsDevice.Viewport.Y + 376), Color.Orange);
            }

            spriteBatch.End();
        }

        private void DrawBackground()
        {
            spriteBatch.Draw(mainBackground, Vector2.Zero, Color.White);
            backgroundLayer1.Draw(spriteBatch);
            backgroundLayer2.Draw(spriteBatch);
        }

        #endregion

    }
}